=== modified file 'bzrlib/bundle/serializer/v4.py'
--- bzrlib/bundle/serializer/v4.py	2008-03-19 04:39:04 +0000
+++ bzrlib/bundle/serializer/v4.py	2008-04-28 00:49:40 +0000
@@ -588,7 +588,10 @@
     def _install_revision(self, revision_id, metadata, text):
         if self._repository.has_revision(revision_id):
             return
-        self._repository._add_revision_text(revision_id, text)
+        if self._info['serializer'] == self._repository._serializer.format_num:
+            self._repository._add_revision_text(revision_id, text)
+        revision = self._source_serializer.read_revision_from_string(text)
+        self._repository.add_revision(revision.revision_id, revision)
 
     def _install_signature(self, revision_id, metadata, text):
         transaction = self._repository.get_transaction()

=== modified file 'bzrlib/tests/test_bundle.py'
--- bzrlib/tests/test_bundle.py	2008-03-26 03:06:52 +0000
+++ bzrlib/tests/test_bundle.py	2008-04-28 00:49:40 +0000
@@ -42,7 +42,7 @@
                            NoSuchFile,)
 from bzrlib.merge import Merge3Merger
 from bzrlib.repofmt import knitrepo
-from bzrlib.osutils import sha_file
+from bzrlib.osutils import sha_file, sha_string
 from bzrlib.tests import (
     SymlinkFeature,
     TestCase,
@@ -1015,19 +1015,29 @@
         self.assertNotContainsRe(inv_text, 'format="5"')
         self.assertContainsRe(inv_text, 'format="7"')
 
+    def make_repo_with_installed_revisions(self):
+        tree = self.make_simple_tree('knit')
+        tree.commit('hello', rev_id='rev1')
+        tree.commit('hello', rev_id='rev2')
+        bundle = read_bundle(self.create_bundle_text('null:', 'rev2')[0])
+        repo = self.make_repository('repo', format='dirstate-with-subtree')
+        bundle.install_revisions(repo)
+        inv = repo.get_inventory('rev2')
+        return repo
+
     def test_across_models(self):
-        tree = self.make_simple_tree('knit')
-        tree.commit('hello', rev_id='rev1')
-        tree.commit('hello', rev_id='rev2')
-        bundle = read_bundle(self.create_bundle_text('null:', 'rev2')[0])
-        repo = self.make_repository('repo', format='dirstate-with-subtree')
-        bundle.install_revisions(repo)
-        inv = repo.get_inventory('rev2')
+        repo = self.make_repo_with_installed_revisions()
         self.assertEqual('rev2', inv.root.revision)
         root_vf = repo.weave_store.get_weave(inv.root.file_id,
                                              repo.get_transaction())
         self.assertEqual(root_vf.versions(), ['rev1', 'rev2'])
 
+    def test_inv_hash_across_serializers(self):
+        repo = self.make_repo_with_installed_revisions()
+        recorded_inv_sha1 = repo.get_inventory_sha1('rev2')
+        xml = repo.get_inventory_xml('rev2')
+        self.assertEqual(sha_string(xml), recorded_inv_sha1)
+
     def test_across_models_incompatible(self):
         tree = self.make_simple_tree('dirstate-with-subtree')
         tree.commit('hello', rev_id='rev1')

